<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXTakeaTour
 * @subpackage CBXTakeaTour/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    CBXTakeaTour
 * @subpackage CBXTakeaTour/admin
 * @author     Codeboxr Team <sabuj@codeboxr.com>
 */
class CBXTakeaTour_Admin {

	private $cbxtakeatour;

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->settings_api = new CBXTakeaTour_Settings( );
	}// End construct

	/**
	 * Init setting
	 */
	public function setting_init() {
		//set the settings
		$this->settings_api->set_sections( $this->get_settings_sections() );
		$this->settings_api->set_fields( $this->get_settings_fields() );
		//initialize settings
		$this->settings_api->admin_init();
	}//end setting_init

	/**
	 * Set settings fields
	 *
	 * @return type array
	 */
	public function get_settings_sections() {
		$sections = array(
			/*array(
				'id'    => 'cbxtakeatour_basics',
				'title' => esc_html__( 'Plugin Options', 'cbxtakeatour' ),
			),*/

			array(
				'id'    => 'cbxtakeatour_tools',
				'title' => esc_html__( 'Tools', 'cbxtakeatour' )
			)
		);

		$sections = apply_filters( 'cbxtakeatour_setting_sections', $sections );

		return $sections;
	}//end get_settings_sections

	/**
	 * Returns all the settings fields
	 *
	 * @return array settings fields
	 */
	public function get_settings_fields() {

		$reset_data_link = add_query_arg( 'cbxtakeatour_fullreset', 1, admin_url( 'edit.php?post_type=cbxtour&page=cbxtakeatoursettings' ) );

		$table_html = '<p><a id="cbxtakeatour_info_trig" href="#">' . esc_html__( 'Show/hide details', 'cbxtakeatour' ) . '</a></p>';
		$table_html .= '<div id="cbxtakeatour_resetinfo" style="display: none;">';

		$table_html .= '<p style="margin-bottom: 15px;" id="cbxtakeatour_plg_gfig_info"><strong>' . esc_html__( 'Following option values created by this plugin(including addon) from WordPress core option table', 'cbxtakeatour' ) . '</strong></p>';


		$option_values = CBXTakeaTourHelper::getAllOptionNames();


		$table_html .= '<table class="widefat widethin cbxtakeatour_table_data">
	<thead>
	<tr>
		<th class="row-title">' . esc_attr__( 'Option Name', 'cbxtakeatour' ) . '</th>
		<th>' . esc_attr__( 'Option ID', 'cbxtakeatour' ) . '</th>		
		<th>' . esc_attr__( 'Option Value', 'cbxtakeatour' ) . '</th>		
	</tr>
	</thead>';

		$table_html .= '<tbody>';

		$i = 0;
		foreach ( $option_values as $key => $value ) {

			$alternate_class = ( $i % 2 == 0 ) ? 'alternate' : '';
			$i ++;
			$table_html .= '<tr class="' . esc_attr( $alternate_class ) . '">
									<td class="row-title"><label for="tablecell">' . esc_attr( $value['option_name']) . '</label></td>
									<td>' . esc_attr( $value['option_id'] ) . '</td>
									<td><code style="overflow-wrap: break-word; word-break: break-all;">' . $value['option_value'] . '</code></td>
								</tr>';
		}

		$table_html .= '</tbody>';
		$table_html .= '<tfoot>
	<tr>
		<th class="row-title">' . esc_attr__( 'Option Name', 'cbxtakeatour' ) . '</th>
		<th>' . esc_attr__( 'Option ID', 'cbxtakeatour' ) . '</th>		
		<th>' . esc_attr__( 'Option Value', 'cbxtakeatour' ) . '</th>		
	</tr>
	</tfoot>
</table>';



		$table_names = CBXTakeaTourHelper::getAllDBTablesList();

		$table_html .= '<p style="margin-bottom: 15px;" id="cbxscratingreview_info"><strong>' . esc_html__( 'Following database tables will be reset/deleted and then re-created.', 'cbxtakeatour' ) . '</strong></p>';

		$table_html .= '<table class="widefat widethin cbxtakeatour_table_data">
	<thead>
	<tr>
		<th class="row-title">' . esc_attr__( 'Table Name', 'cbxtakeatour' ) . '</th>
		<th>' . esc_attr__( 'Table Name in DB', 'cbxtakeatour' ) . '</th>		
	</tr>
	</thead>';

		$table_html .= '<tbody>';


		$i = 0;
		foreach ( $table_names as $key => $value ) {
			$alternate_class = ( $i % 2 == 0 ) ? 'alternate' : '';
			$i ++;
			$table_html .= '<tr class="' . esc_attr( $alternate_class ) . '">
									<td class="row-title"><label for="tablecell">' . esc_attr( $key) . '</label></td>
									<td>' . esc_attr( $value ) . '</td>									
								</tr>';
		}

		$table_html .= '</tbody>';
		$table_html .= '<tfoot>
	<tr>
		<th class="row-title">' . esc_attr__( 'Table Name', 'cbxtakeatour' ) . '</th>
		<th>' . esc_attr__( 'Table Name in DB', 'cbxtakeatour' ) . '</th>		
	</tr>
	</tfoot>
</table>';
		$table_html .= '</div>';
	    

		$settings_fields = array(
			/*'cbxtakeatour_ga' => array(
				array(
					'name'    => 'enabled',
					'label'   => esc_html__( 'Google Enalytics Tracking', 'cbxtakeatour' ),
					'desc'    => esc_html__( 'Enable/Disable. You must have google analytics added to your website to use this feature.', 'cbxtakeatour' ),
					'type'    => 'checkbox',
					'default' => 'off',
				),
				array(
					'name'    => 'track_tour_start',
					'label'   => esc_html__( 'Track Tour Start', 'cbxtakeatour' ),
					'desc'    => esc_html__( 'Enable/Disable. Track tour start click.', 'cbxtakeatour' ),
					'type'    => 'checkbox',
					'default' => 'on',
				),
				array(
					'name'    => 'track_pbr',
					'label'   => esc_html__( 'Prevent Bounce Rate', 'cbxtakeatour' ),
					'desc'    => esc_html__( 'Enable/Disable. Turn it on to indicate that the event hit will not be used in bounce-rate calculation.', 'cbxtakeatour' ),
					'type'    => 'checkbox',
					'default' => 'on',
				),
			),*/
			'cbxtakeatour_tools' => array(
				'delete_global_config' => array(
					'name'     => 'delete_global_config',
					'label'    => esc_html__( 'On Uninstall delete plugin data', 'cbxtakeatour' ),
					'desc'     => '<p>' . __( 'Delete Global Config data and custom table created by this plugin on uninstall.', 'cbxtakeatour' ) . ' ' . __( 'Details table information is <a href="#cbxtakeatour_plg_gfig_info">here</a>', 'cbxtakeatour' ) . '</p>' . '<p>' . __( '<strong>Please note that this process can not be undone and it is recommended to keep full database backup before doing this.</strong>', 'cbxtakeatour' ) . '</p>',
					'type'     => 'radio',
					'options'  => array(
						'yes' => esc_html__( 'Yes', 'cbxtakeatour' ),
						'no'  => esc_html__( 'No', 'cbxtakeatour' ),
					),
					'default'  => 'no'
				),
				'reset_data'           => array(
					'name'     => 'reset_data',
					'label'    => esc_html__( 'Reset all data', 'cbxtakeatour' ),
					'desc'     => sprintf( __( 'Reset option values and all tables created by this plugin. 
<a class="button button-primary" onclick="return confirm(\'%s\')" href="%s">Reset Data</a>', 'cbxtakeatour' ), esc_html__( 'Are you sure to reset all data, this process can not be undone?', 'cbxtakeatour' ), $reset_data_link ) . $table_html,
					'type'     => 'html',
					'default'  => 'off'
				)
			)
		);

		$settings_fields = apply_filters( 'cbxtakeatour_setting_fields', $settings_fields );

		return $settings_fields;
	}//end get_settings_fields

	/**
	 * Create admin menu's
	 */
	public function admin_pages() {

		$page = isset( $_GET['page'] ) ? esc_attr( wp_unslash( $_GET['page'] ) ) : '';

		//review listing page
		/*$email_logger_menu_hook = add_menu_page( esc_html__( 'CBX SMTP and Email Logger Dashboard', 'cbxwpemaillogger' ),
			esc_html__( 'CBX SMTP & Logs', 'cbxwpemaillogger' ),
			'manage_options',
			'cbxwpemaillogger',
			array( $this, 'display_cbxwpemaillogger_listing_page' ),
			CBXWPEMAILLOGGER_ROOT_URL . 'assets/images/icon_20.png',
			'6' );

		//add screen option save option
		if ( $page == 'cbxwpemaillogger' ) {
			add_action( "load-$email_logger_menu_hook", array( $this, 'cbxwpemaillogger_listing_perpage' ) );
		}*/

		//add settings for this plugin
		$setting_page_hook = add_submenu_page( 'edit.php?post_type=cbxtour',
			esc_html__( 'Global Setting', 'cbxtakeatour' ),
			esc_html__( 'Global Setting', 'cbxtakeatour' ),
			'manage_options',
			'cbxtakeatoursettings',
			array( $this, 'display_plugin_admin_settings' ) );

		//add settings for this plugin
		/*$testmail_page_hook = add_submenu_page( 'cbxwpemaillogger',
			esc_html__( 'Email Testing', 'cbxwpemaillogger' ),
			esc_html__( 'Email Testing', 'cbxwpemaillogger' ),
			'manage_options',
			'cbxwpemailloggeremailtesting',
			array( $this, 'display_plugin_admin_email_testing' ) );*/

		/*global $submenu;
		if ( isset( $submenu['cbxwpemaillogger'][0][0] ) ) {
			$submenu['cbxwpemaillogger'][0][0] = esc_html__( 'Email Logs', 'cbxwpemaillogger' );
		}*/


	}//end admin_pages

	/**
	 * Display settings page
	 *
	 * @global type $wpdb
	 */
	public function display_plugin_admin_settings() {
		global $wpdb;

		//$plugin_data = get_plugin_data( plugin_dir_path( __DIR__ ) . '/../' . $this->plugin_basename );

		echo cbxtakeatour_get_template_html( 'admin/admin-settings-display.php', array( 'ref' => $this, 'settings_api' => $this->settings_api ) );
	}//end display_plugin_admin_settings

	/**
	 * Set default title for tour
	 */
	public function default_title_extend() {
		global $post_type;
		if ( 'cbxtour' == $post_type ) {
			return esc_html__( 'Feature Tour Example', 'cbxtakeatour' );
		}
	}//end default_title_extend

	/**
	 * Remove monthly dropdown for cbxtour post listing in admin
	 *
	 * @param $months
	 * @param $post_type
	 *
	 * @return array
	 */
	public function months_dropdown_results_extend( $months, $post_type ) {
		if ( 'cbxtour' == $post_type ) {
			return array();
		}

		return $months;
	}//end months_dropdown_results_extend

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles( $hook ) {
		global $post_type, $post;

		$page = isset( $_GET['page'] ) ? esc_attr( wp_unslash( $_GET['page'] ) ) : '';

		//if ( ( $hook == 'post.php' || $hook == 'post-new.php' || $hook == 'edit.php' ) && $post_type == 'cbxtour' && $page != 'cbxtoursettings' ) {
		if ( ( $hook == 'post.php' || $hook == 'post-new.php' || $hook == 'edit.php' ) && $post_type == 'cbxtour' ) {
			wp_register_style( 'ply',
				plugin_dir_url( __FILE__ ) . '../assets/vendor/ply/Ply.css',
				array(),
				$this->version,
				'all' );
			wp_register_style( 'minitoggle',
				plugin_dir_url( __FILE__ ) . '../assets/css/minitoggle.css',
				array(),
				$this->version,
				'all' );
			wp_register_style( 'bootstrap-tour-standalone',
				plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone.css',
				array(),
				$this->version,
				'all' );
			wp_register_style( 'cbxtakeatour-admin',
				plugin_dir_url( __FILE__ ) . '../assets/css/cbxtakeatour-admin.css',
				array( 'editor-buttons', 'ply', 'bootstrap-tour-standalone' ),
				$this->version,
				'all' );

			wp_enqueue_style( 'editor-buttons' );
			wp_enqueue_style( 'ply' );
			wp_enqueue_style( 'wp-color-picker' );
			wp_enqueue_style( 'minitoggle' );
			wp_enqueue_style( 'bootstrap-tour-standalone' );
			wp_enqueue_style( 'cbxtakeatour-admin' );
		}



		if ( $page == 'cbxtakeatoursettings') {
			wp_enqueue_style( 'wp-color-picker' );


			wp_register_style( 'select2',plugin_dir_url( __FILE__ ) . '../assets/js/select2/css/select2.min.css',
				array(),
				$this->version );
			wp_register_style( 'cbxtakeatour-setting',
				plugin_dir_url( __FILE__ ) . '../assets/css/cbxtakeatour-setting.css',
				array(
					'select2',
					'wp-color-picker',
				),
				$this->version,
				'all' );

			wp_enqueue_style( 'select2' );
			wp_enqueue_style( 'cbxtakeatour-setting' );
		}
	}// End enqueue_styles


	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts( $hook ) {
		global $post_type, $post;

		$page = isset( $_GET['page'] ) ? esc_attr( wp_unslash( $_GET['page'] ) ) : '';

		//if ( ( $hook == 'post.php' || $hook == 'post-new.php' || $hook == 'edit.php' ) && $post_type == 'cbxtour' && $page != 'cbxtoursettings' ) {
		if ( ( $hook == 'post.php' || $hook == 'post-new.php' || $hook == 'edit.php' ) && $post_type == 'cbxtour' ) {

			wp_register_script( 'ply',
				plugin_dir_url( __FILE__ ) . '../assets/vendor/ply/Ply.min.js',
				array( 'jquery' ),
				$this->version,
				true );

			wp_register_script( 'wp-color-picker-alpha',
				plugin_dir_url( __FILE__ ) . '../assets/js/alpha-color-picker.js',
				array( 'jquery', 'wp-color-picker' ),
				$this->version,
				true );

			wp_register_script( 'minitoggle',
				plugin_dir_url( __FILE__ ) . '../assets/js/minitoggle.js',
				array( 'jquery' ),
				$this->version,
				true );

			wp_register_script( 'cbxtakeatour-admin',
				plugin_dir_url( __FILE__ ) . '../assets/js/cbxtakeatour-admin.js',
				array( 'jquery', 'editor', 'ply', 'wp-color-picker-alpha', 'minitoggle' ),
				$this->version,
				true );


			$translation_placeholder = apply_filters( 'cbxtakeatour-admin-vars',
				array(
					'teeny_setting' => array(
						'teeny'         => true,
						'media_buttons' => true,
						'editor_class'  => '',
						'textarea_rows' => 5,
						'quicktags'     => false,
						'menubar'       => false,
					),
					'confirm_msg'   => esc_html__( 'Are you sure want to delete this ?', 'cbxtakeatour' ),
					'confirm_yes'   => esc_html__( 'Yes', 'cbxtakeatour' ),
					'confirm_no'    => esc_html__( 'No', 'cbxtakeatour' ),
				) );

			wp_localize_script( 'cbxtakeatour-admin', 'cbxtakeatour_admin', $translation_placeholder );


			if ( ! class_exists( '_WP_Editors', false ) ) {
				require( ABSPATH . WPINC . '/class-wp-editor.php' );
			}
			\_WP_Editors::print_tinymce_scripts();

			wp_enqueue_media();
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'editor' );
			wp_enqueue_script( 'ply' );
			wp_enqueue_script( 'wp-color-picker' );
			wp_enqueue_script( 'wp-color-picker-alpha' );
			wp_enqueue_script( 'minitoggle' );
			wp_enqueue_script( 'cbxtakeatour-admin' );
		}

		if ( $page == 'cbxtakeatoursettings' ) {
			wp_enqueue_script( 'jquery' );

			wp_enqueue_media();
			wp_enqueue_script( 'wp-color-picker' );

			wp_register_script( 'select2', plugin_dir_url( __FILE__ ) . '../assets/js/select2/js/select2.min.js', array( 'jquery' ), $this->version, true );
			wp_register_script( 'cbxtakeatour-setting', plugin_dir_url( __FILE__ ) . '../assets/js/cbxtakeatour-setting.js',
				array(
					'jquery',
					'select2',
					'wp-color-picker',
				),
				$this->version,
				true );


			$cbxtakeatour_admin_js_vars = apply_filters( 'cbxtakeatour_admin_js_vars',
				array(
					'ajaxurl'          => admin_url( 'admin-ajax.php' ),
					'nonce'            => wp_create_nonce( "cbxtakeatournonce" ),
					'please_select'    => esc_html__( 'Please Select', 'cbxtakeatour' ),
					'upload_title'     => esc_html__( 'Window Title', 'cbxtakeatour' ),
					//'cbxbookmark_lang' => get_user_locale(),
				) );

			wp_localize_script( 'cbxtakeatour-setting', 'cbxtakeatour_setting', $cbxtakeatour_admin_js_vars );

			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'select2' );
			wp_enqueue_script( 'cbxtakeatour-setting' );
		}

	}// End enqueue_scripts


	// Register Custom Post Type
	public function create_tour() {

		$labels = array(
			'name'               => _x( 'CBX Tours', 'Post Type General Name', 'cbxtakeatour' ),
			'singular_name'      => _x( 'CBX Tour', 'Post Type Singular Name', 'cbxtakeatour' ),
			'menu_name'          => esc_html__( 'CBX Tour', 'cbxtakeatour' ),
			'parent_item_colon'  => esc_html__( 'Parent Tour:', 'cbxtakeatour' ),
			'all_items'          => esc_html__( 'All Tours', 'cbxtakeatour' ),
			'view_item'          => esc_html__( 'View Tour', 'cbxtakeatour' ),
			'add_new_item'       => esc_html__( 'Add New Tour', 'cbxtakeatour' ),
			'add_new'            => esc_html__( 'Add New', 'cbxtakeatour' ),
			'edit_item'          => esc_html__( 'Edit Tour', 'cbxtakeatour' ),
			'update_item'        => esc_html__( 'Update Tour', 'cbxtakeatour' ),
			'search_items'       => esc_html__( 'Search Tour', 'cbxtakeatour' ),
			'not_found'          => esc_html__( 'No tours available, please create one.', 'cbxtakeatour' ),
			'not_found_in_trash' => esc_html__( 'Not found in Trash', 'cbxtakeatour' ),
		);
		$args   = array(
			'label'               => esc_html__( 'cbxtour', 'cbxtakeatour' ),
			'description'         => esc_html__( 'Tour steps', 'cbxtakeatour' ),
			'labels'              => $labels,
			'supports'            => array( 'title', 'custom-fields' ),
			'hierarchical'        => false,
			'public'              => true,
			'show_ui'             => true,
			'show_in_menu'        => true, //left dashboard menu
			'show_in_nav_menus'   => false, //menu create interface
			'show_in_admin_bar'   => true, //top admin bar menu
			'menu_position'       => 5,
			//'menu_icon'           => 'dashicons-megaphone',
			'menu_icon'           => CBXTAKEATOUR_ROOT_URL . 'assets/images/icon_w_24.png',
			'can_export'          => true,
			'has_archive'         => false,
			'exclude_from_search' => true,
			'publicly_queryable'  => false,
			'capability_type'     => 'post',
		);
		register_post_type( 'cbxtour', $args );

	}// End create_tour


	/**
	 * cbxtour type post listing extra cols
	 *
	 * @param $cbxtour_columns
	 *
	 * @return mixed
	 */
	public function add_new_tour_columns( $cbxtour_columns ) {
		$cbxtour_columns['shortcode'] = esc_html__( 'Shortcode', 'cbxtakeatour' );

		if ( isset( $cbxtour_columns['date'] ) ) {
			unset( $cbxtour_columns['date'] );
		}

		$customOrder = array( 'cb', 'title', 'shortcode' );

		# return a new column array to wordpress.
		# order is the exactly like you set in $customOrder.
		$cbxtour_columns_new = array();

		foreach ( $customOrder as $colname ) {
			$cbxtour_columns_new[ $colname ] = $cbxtour_columns[ $colname ];
		}

		return $cbxtour_columns_new;
	}// End add_new_tour_columns


	/**
	 * cbxtour type post listing extra col values
	 */
	public function manage_tour_columns() {

		global $post;
		$post_id = $post->ID;

		echo '<span title="' . esc_html__( "Click to clipboard",
				"cbxtakeatour" ) . '" id="cbxtakeatourshortcode-' . $post_id . '" class="cbxtakeatourshortcode cbxtakeatourshortcode-' . $post_id . '">[cbxtakeatour id="' . $post_id . '"]</span>';

	}// End manage_poll_columns


	/**
	 * This method add meta boxes to the tour plugin
	 */
	public function add_meta_boxes() {
		add_meta_box(
			'cbxtourmetabox',
			esc_html__( 'Tour Steps', 'cbxtakeatour' ),
			array( $this, 'cbxtourmetabox_display' ),
			'cbxtour',
			'normal',
			'high'
		);
		//add meta box in right col to show the result
		add_meta_box(
			'cbxtourmetabox_preview_meta_box',
			esc_html__( 'Preview', 'cbxtakeatour' ),
			array( $this, 'cbxtourmetabox_preview_display' ),
			'cbxtour',
			'side',
			'high'
		);

		add_meta_box(
			'cbxtourmetabox_shortcode_meta_box',
			esc_html__( 'Shortcode', 'cbxtakeatour' ),
			array( $this, 'cbxtourmetabox_shortcode_display' ),
			'cbxtour',
			'side',
			'low'
		);
	} // End add_meta_boxes


	/**
	 * This method Displays meta boxes sections and options
	 */
	public function cbxtourmetabox_display( $post ) {
		$post_id     = intval( $post->ID );
		$fieldValues = get_post_meta( $post_id, '_cbxtourmeta', true );

		$layout     = isset( $fieldValues['layout'] ) ? esc_attr( wp_unslash( $fieldValues['layout'] ) ) : 'basic';
		$auto_start = isset( $fieldValues['auto_start'] ) ? intval( $fieldValues['auto_start'] ) : 0;

		$display      = isset( $fieldValues['display'] ) ? intval( $fieldValues['display'] ) : 1;
		$redirect_url = ( isset( $fieldValues['redirect_url'] ) && $fieldValues['redirect_url'] != '' ) ? esc_url( $fieldValues['redirect_url'] ) : '';

		$tour_button_text = isset( $fieldValues['tour_button_text'] ) ? esc_attr( wp_unslash( $fieldValues['tour_button_text'] ) ) : esc_html__( 'Take a tour',
			'cbxtakeatour' );

		$tour_button_block = isset( $fieldValues['tour_button_block'] ) ? intval( $fieldValues['tour_button_block'] ) : 0;
		$tour_button_align = isset( $fieldValues['tour_button_align'] ) ? esc_attr( wp_unslash( $fieldValues['tour_button_align'] ) ) : 'center';


		$cbxtour_meta_tabs = apply_filters( 'cbxtour_meta_tabs',
			array(
				'cbxtakeatour_steps'   => esc_html__( 'Tour Steps', 'cbxtakeatour' ),
				'cbxtakeatour_setting' => esc_html__( 'Tour Settings', 'cbxtakeatour' ),
			),
			$post_id );

		//nonce security field
		wp_nonce_field( 'cbxtourmetabox', 'cbxtourmeta[nonce]' );

		//nav-tab-active
		echo '<div class="tour_postbox">';
		echo '<h2 class="nav-tab-wrapper">';
		foreach ( $cbxtour_meta_tabs as $meta_tab_key => $meta_tab_title ) {
			echo '<a href="#' . esc_attr( $meta_tab_key ) . '" class="nav-tab" id="' . esc_attr( $meta_tab_key ) . '_tab">' . esc_attr( $meta_tab_title ) . '</a>';
		}
		echo '</h2>';

		echo '<div class="metabox-holder takeatour_meta_holder">';

		do_action( 'cbxtour_meta_tab_content_start', $post_id, $post, $fieldValues );
		?>

        <div id="cbxtakeatour_steps" class="cbxtakeatour_group" style="display: none;">
			<?php do_action( 'cbxtour_meta_tab_content_steps_start', $post_id, $post, $fieldValues ); ?>
			<?php
			$totalEntries = ( ( ! empty( $fieldValues['steps'] ) ) ? count( $fieldValues['steps'] ) : 0 );

			echo '<div id="cbxtourmetabox_entries">';

			for ( $i = 0; $i < $totalEntries; $i ++ ) {
				$element   = isset( $fieldValues['steps'][ $i ]['element'] ) ? esc_attr( $fieldValues['steps'][ $i ]['element'] ) : '';
				$title     = isset( $fieldValues['steps'][ $i ]['title'] ) ? esc_html( $fieldValues['steps'][ $i ]['title'] ) : '';
				$content   = isset( $fieldValues['steps'][ $i ]['content'] ) ? $fieldValues['steps'][ $i ]['content'] : '';
				$placement = isset( $fieldValues['steps'][ $i ]['placement'] ) ? esc_attr( $fieldValues['steps'][ $i ]['placement'] ) : '';
				$backdrop  = isset( $fieldValues['steps'][ $i ]['backdrop'] ) ? intval( $fieldValues['steps'][ $i ]['backdrop'] ) : 0;

				?>

                <div class="cbxtourmetabox_entry cbxtourmetabox_entry-<?php echo $i; ?>"
                     data-entry-no="<?php echo $i; ?>">
                    <div class="cbxtourmetabox_step_heading"><span
                                class="dashicons dashicons-editor-justify draggable"></span>
                        <h3 class="step_heading_title"
                            style="display: inline"><?php esc_html_e( 'Step', 'cbxtakeatour' ); ?>
                            #<?php echo $i + 1; ?>
                            : <?php echo $title; ?></h3><a href="#"
                                                           class="dashicons dashicons-post-trash delete-step"
                                                           title="<?php esc_html_e( 'Delete Step',
							                                   'cbxtakeatour' ) ?>"></a>
                    </div>

                    <div class="cbxtourmetabox_wrap">
                        <table class="form-table">
                            <tbody>
                            <tr valign="top">
                                <th scope="row"><label
                                            for="cbxtourmetabox_fields_element_<?php echo $i; ?>"><?php esc_html_e( 'Element',
											'cbxtakeatour' ) ?></label>
                                </th>
                                <td>
                                    <input id="cbxtourmetabox_fields_element_<?php echo $i; ?>"
                                           class="regular-text" autocomplete="new-password"
                                           type="text" name="cbxtourmeta[steps][<?php echo $i; ?>][element]"
                                           placeholder="<?php esc_html_e( '#html_element_id',
										       'cbxtakeatour' ) ?>"
                                           value="<?php echo $element; ?>"/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><label
                                            for="cbxtourmetabox_fields_title_<?php echo $i; ?>"><?php esc_html_e( 'Title',
											'cbxtakeatour' ) ?></label>
                                </th>
                                <td>
                                    <input id="cbxtourmetabox_fields_title_<?php echo $i; ?>"
                                           class="regular-text" autocomplete="new-password"
                                           type="text" name="cbxtourmeta[steps][<?php echo $i; ?>][title]"
                                           placeholder="<?php esc_html_e( 'Title', 'cbxtakeatour' ) ?>"
                                           value="<?php echo $title; ?>"/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><label
                                            for="cbxtourmetabox_fields_content_<?php echo $i; ?>"><?php esc_html_e( 'Content',
											'cbxtakeatour' ) ?></label></th>
                                <td>
									<?php
									$editor_id = 'cbxtourmetabox_fields_content_' . $i;
									$setting   = array(
										'textarea_name' => 'cbxtourmeta[steps][' . $i . '][content]',
										'teeny'         => true,
										'media_buttons' => true,
										'editor_class'  => '',
										'textarea_rows' => 5,
										'quicktags'     => false,
										'menubar'       => false,
									);
									wp_editor( $content, $editor_id, $setting );
									?>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row"><label
                                            for="cbxtourmetabox_fields_placement_<?php echo $i; ?>"><?php esc_html_e( 'Placement',
											'cbxtakeatour' ) ?></label>
                                </th>
                                <td>
									<?php
									$placement_array = CBXTakeaTourHelper::cbxtakeatour_placement();
									foreach ( $placement_array as $key => $value ) {
										?>
                                        <input
                                        id="cbxtourmetabox_fields_placement_<?php echo $i; ?>_<?php echo esc_attr( $key ); ?>"
                                        class=""
                                        type="radio" name="cbxtourmeta[steps][<?php echo $i; ?>][placement]"
                                        value="<?php esc_attr_e( $key ); ?>" <?php checked( $placement,
											$key,
											true ); ?>/><?php echo $value ?>
									<?php } ?>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row"><label
                                            for="cbxtourmetabox_fields_backdrop_<?php echo $i; ?>"><?php esc_html_e( 'Backdrop ',
											'cbxtakeatour' ) ?></label>
                                </th>
                                <td>
									<?php
									$backdrop_array = CBXTakeaTourHelper::cbxtakeatour_bool_arr();
									foreach ( $backdrop_array as $key => $value ) {
										?>
                                        <input id="cbxtourmetabox_fields_backdrop_<?php echo $i; ?>-<?php echo $key; ?>"
                                               class=""
                                               type="radio"
                                               name="cbxtourmeta[steps][<?php echo $i; ?>][backdrop]"
                                               value="<?php echo intval( $key ); ?>" <?php checked( $backdrop,
											$key,
											true ); ?> /> <?php echo $value ?>
									<?php }; ?>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </div>
                </div>

				<?php
			}//end loop
			echo '</div>';

			echo '<p style="text-align:center;"><a data-counter="' . $totalEntries . '" id="cbxtourmetaboxaddstep" class="button-primary" href="#">' . esc_html__( 'Add Step',
					'cbxtakeatour' ) . '</a> <a ' . ( ( $totalEntries == 0 ) ? ' style="display: none;" ' : '' ) . ' id="cbxtourmetaboxremstep" class="button-primary" href="#">' . esc_html__( 'Remove All',
					'cbxtakeatour' ) . '</a></p>';

			$template = '
<div class="cbxtourmetabox_entry cbxtourmetabox_entry-COUNTER" data-entry-no="COUNTER">
	<div class="cbxtourmetabox_step_heading"><span class="dashicons dashicons-editor-justify draggable"></span>
	<h3 class="step_heading_title" style="display: inline">' . esc_html__( 'Step',
					'cbxtakeatour' ) . '#COUNTERPLUS : ' . esc_html__( "Untitled",
					"cbxtakeatour" ) . '</h3><a href="#" class="dashicons dashicons-post-trash delete-step" title="' . esc_html__( "Delete Step",
					"cbxtakeatour" ) . '"></a></div>
	<div class="cbxtourmetabox_wrap">
		<table class="form-table">
			<tbody>
				<tr valign="top">
					<th scope="row"><label for="cbxtourmetabox_fields_element_COUNTER">' . esc_html__( 'Element',
					'cbxtakeatour' ) . '</label></th>
					<td>
						<input id="cbxtourmetabox_fields_element_COUNTER" autocomplete="new-password" class="regular-text" type="text" name="cbxtourmeta[steps][COUNTER][element]" placeholder="' . esc_html__( '#HTML element id',
					'cbxtakeatour' ) . '" value="">
					</td>
				</tr>
				<tr valign="top">
					<th scope="row"><label for="cbxtourmetabox_fields_title_COUNTER">' . esc_html__( 'Title',
					'cbxtakeatour' ) . '</label></th>
					<td>
						<input id="cbxtourmetabox_fields_title_COUNTER" autocomplete="new-password" class="regular-text" type="text" name="cbxtourmeta[steps][COUNTER][title]" placeholder="' . esc_html__( 'Title',
					'cbxtakeatour' ) . '" value="Untitled">
					</td>
				</tr>
				<tr valign="top">
					<th scope="row"><label for="cbxtourmetabox_fields_content_COUNTER">' . esc_html__( 'Content',
					'cbxtakeatour' ) . '</label></th>
					<td>
						<textarea class="all-options" id="cbxtourmetabox_fields_content_COUNTER" name="cbxtourmeta[steps][COUNTER][content]" placeholder="' . esc_html__( 'Content',
					'cbxtakeatour' ) . '"></textarea>
					</td>
				</tr>
                <tr valign="top">
					<th scope="row"><label for="cbxtourmetabox_fields_placement_COUNTER">' . esc_html__( 'Placement',
					'cbxtakeatour' ) . '</label></th>
					<td>
					    <input id="cbxtourmetabox_fields_placement_COUNTER_top" type="radio" name="cbxtourmeta[steps][COUNTER][placement]" value="top">' . esc_html__( 'Top',
					'cbxtakeatour' ) . '
					    <input id="cbxtourmetabox_fields_placement_COUNTER_bottom" type="radio" name="cbxtourmeta[steps][COUNTER][placement]" value="bottom" checked>' . esc_html__( 'Bottom',
					'cbxtakeatour' ) . '
					    <input id="cbxtourmetabox_fields_placement_COUNTER_left" type="radio" name="cbxtourmeta[steps][COUNTER][placement]" value="left">' . esc_html__( 'Left',
					'cbxtakeatour' ) . '
					    <input id="cbxtourmetabox_fields_placement_COUNTER_right" type="radio" name="cbxtourmeta[steps][COUNTER][placement]" value="right">' . esc_html__( 'Right',
					'cbxtakeatour' ) . '
                  </td>
				</tr>
				<tr valign="top">
					<th scope="row"><label for="cbxtourmetabox_fields_backdrop_COUNTER">' . esc_html__( 'Backdrop',
					'cbxtakeatour' ) . '</label></th>
					<td>
					    <input id="cbxtourmetabox_fields_backdrop_COUNTER-1" type="radio" name="cbxtourmeta[steps][COUNTER][backdrop]"  value="1">' . esc_html__( 'Yes',
					'cbxtakeatour' ) . '
					    <input id="cbxtourmetabox_fields_backdrop_COUNTER-0" type="radio" name="cbxtourmeta[steps][COUNTER][backdrop]" checked value="0">' . esc_html__( 'No',
					'cbxtakeatour' ) . '
                  </td>
				</tr>
			</tbody>
		</table>
	</div>
</div>';

			?>
            <script type="text/javascript">
                var elementHTMLStep = <?php echo json_encode( $template ); ?>;

            </script>
			<?php
			echo '<br><label style="font-weight: bold" for="cbxtourmetabox_fields_redirect_url">' . esc_html__( "Redirect Url : ",
					"cbxtakeatour" ) . '</label><input name="cbxtourmeta[redirect_url]" placeholder="https://example.com" id="cbxtourmetabox_fields_redirect_url" type="text" class="regular-text" value="' . $redirect_url . '"><p class="cbxtakeatoururldec">' . esc_html__( "The URL link will redirect to the end of the tour.",
					"cbxtakeatour" ) . '</p></div>';
			?>
			<?php do_action( 'cbxtour_meta_tab_content_steps_end', $post_id, $post, $fieldValues ); ?>
        </div>
        <div id="cbxtakeatour_setting" class="cbxtakeatour_group" style="display: none;">
	        <?php do_action( 'cbxtour_meta_tab_content_setting_start', $post_id, $post, $fieldValues ); ?>
            <table class="form-table">
                <tbody>
                <?php
                do_action('cbxtour_meta_tab_content_setting_start_fields', $post_id, $post, $fieldValues);
                ?>
                <tr valign="top">
                    <th scope="row">
                        <label for="cbxtourmetabox_fields_display">
							<?php esc_html_e( 'Display tour button', 'cbxtakeatour' ) ?>
                        </label>
                    </th>
                    <td>
                        <!--<div id="display_toggle" class="minitoggle_trigger_display" style="width: 55px;cursor: pointer"></div>-->
                        <input id="cbxtourmetabox_fields_display" class="minitoggle_trigger"
                               type="hidden" name="cbxtourmeta[display]"
                               value="<?php echo intval( $display ); ?>" <?php checked( $display,
							true ); ?> />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label
                                for="cbxtourmetabox_fields_autostart"><?php esc_html_e( 'Auto-start',
								'cbxtakeatour' ) ?></label>
                    </th>
                    <td>
                        <!--<div id="autostart_toggle" class="minitoggle_trigger_display" style="width: 55px;cursor: pointer"></div>-->
                        <input id="cbxtourmetabox_fields_autostart"
                               class="minitoggle_trigger"
                               type="hidden" name="cbxtourmeta[auto_start]"
                               value="<?php echo intval( $auto_start ); ?>" <?php checked( $auto_start,
							true ); ?> />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label
                                for="cbxtourmetabox_fields_tour_button_block"><?php esc_html_e( 'Block Button(Full Width)',
								'cbxtakeatour' ) ?></label>
                    </th>
                    <td>
                        <div id="block_toggle" style="width: 55px;cursor: pointer"></div>
                        <input id="cbxtourmetabox_fields_tour_button_block"
                               class=""
                               type="hidden" name="cbxtourmeta[tour_button_block]"
                               value="<?php echo intval( $tour_button_block ); ?>" <?php checked( $tour_button_block,
							true ); ?> />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label
                                for="cbxtourmetabox_fields_tour_button_align"><?php esc_html_e( 'Button Align',
								'cbxtakeatour' ) ?></label>
                    </th>
                    <td>
                        <select id="cbxtourmetabox_fields_tour_tour_button_align" class=""
                                name="cbxtourmeta[tour_button_align]">
                            <option <?php selected( $tour_button_align, 'center' ); ?>
                                    value="center"><?php esc_html_e( 'Center', 'cbxtakeatour' ); ?></option>
                            <option <?php selected( $tour_button_align, 'left' ); ?>
                                    value="left"><?php esc_html_e( 'Left', 'cbxtakeatour' ); ?></option>
                            <option <?php selected( $tour_button_align, 'right' ); ?>
                                    value="right"><?php esc_html_e( 'Right', 'cbxtakeatour' ); ?></option>
                            <option <?php selected( $tour_button_align, 'none' ); ?>
                                    value="none"><?php esc_html_e( 'N/A', 'cbxtakeatour' ); ?></option>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label
                                for="cbxtourmetabox_fields_tour_button_text"><?php esc_html_e( 'Tour Button text',
								'cbxtakeatour' ) ?></label>
                    </th>
                    <td>
                        <input id="cbxtourmetabox_fields_tour_button_text"
                               class="regular-text"
                               type="text" name="cbxtourmeta[tour_button_text]"
                               value="<?php echo esc_attr( $tour_button_text ); ?>"/>
                    </td>
                </tr>
                <tr>
                    <td colspan="2" style="padding-right: 0px; padding-left: 0px; font-weight: bold">
						<?php esc_html_e( 'Select Tour layout (Need to save the post)',
							'cbxtakeatour' ); ?>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="cbxtourmetabox_layout"><?php esc_html_e( 'Layout',
								'cbxtakeatour' ) ?></label></th>
                    <td>
						<?php
						$layout_options = CBXTakeaTourHelper::cbxtakeatour_layouts();

						echo '<select class="regular-text form-control-lg" name="cbxtourmeta[layout]" id="cbxtourmetabox_layout">';
						foreach ( $layout_options as $layout_key => $option_name ) {
							echo sprintf( '<option value="%s" ' . selected( $layout,
									$layout_key,
									false ) . ' >%s</option>',
								esc_attr( $layout_key ),
								esc_attr( $option_name ) );

						}
						echo '</select>';
						?>
                    </td>
                </tr>
                <?php
                do_action('cbxtour_meta_tab_content_setting_end_fields', $post_id, $post, $fieldValues);
                ?>
                </tbody>
            </table>
			<?php do_action( 'cbxtour_meta_tab_content_setting_end', $post_id, $post, $fieldValues ); ?>
        </div>

		<?php
		do_action( 'cbxtour_meta_tab_content_end', $post_id, $post, $fieldValues );
		echo '</div>'; //.takeatour_meta_holder

		echo '</div>'; // end of div postbox
	}//End cbxtourmetabox_display


	/**
	 * Renders metabox in right col to show  shortcode with copy to clipboard
	 */
	function cbxtourmetabox_shortcode_display() {
		global $post;
		$post_id = $post->ID;

		echo '<span data-clipboard-text=\'[cbxtakeatour id="' . intval( $post_id ) . '"]\' title="' . esc_html__( "Click to clipboard",
				"cbxtakeatour" ) . '" id="cbxtakeatourshortcode-' . intval( $post_id ) . '" class="cbxtakeatourshortcode cbxtakeatourshortcode-edit cbxtakeatourshortcode-single cbxtakeatourshortcode-' . intval( $post_id ) . '">[cbxtakeatour id="' . intval( $post_id ) . '"]</span>';
		echo '<div class="cbxtourclear"></div>';
	}//End cbxtourmetabox_shortcode_display


	/**
	 * Show preview Tour button & box
	 */
	public function cbxtourmetabox_preview_display() {
		global $post;
		$post_id     = intval( $post->ID );
		$fieldValues = get_post_meta( $post_id, '_cbxtourmeta', true );

		$layout           = isset( $fieldValues['layout'] ) ? esc_attr( wp_unslash( $fieldValues['layout'] ) ) : 'basic';
		$tour_button_text = ( isset( $fieldValues['tour_button_text'] ) && $fieldValues['tour_button_text'] != '' ) ? esc_attr( wp_unslash( $fieldValues['tour_button_text'] ) ) : esc_html__( 'Take a tour',
			'cbxtakeatour' );


		$layouts = array_keys( CBXTakeaTourHelper::cbxtakeatour_layouts() );

		if ( ! in_array( $layout, $layouts ) ) {
			$layout = 'basic';
		}

		$layout_class = 'cbxtakeatour_popover_' . $layout;
		$button_class = 'cbxtakeatour-btn-' . $layout;

		$ready_layouts = CBXTakeaTourHelper::cbxtakeatour_layout_ready_styles();
		if ( array_key_exists( $layout, $ready_layouts ) ) {
			$custom_css = CBXTakeaTourHelper::add_custom_css( $post_id, CBXTakeaTourHelper::cbxtakeatour_layout_ready_style( $layout ) );

			wp_register_style( 'cbxtakeatour-admin-inline', false, array( 'cbxtakeatour-admin' ) );
			wp_enqueue_style( 'cbxtakeatour-admin-inline' );
			wp_add_inline_style( 'cbxtakeatour-admin-inline', $custom_css );
		}

		do_action( 'cbxtakeatour_display_tour_admin_enqueue', $layout, $post_id, $fieldValues );
		?>
        <div class="cbxtakeatour-prview">
            <div class="cbxtakeatour_button">
                <h3><?php esc_html_e( 'Tour Button/Link Preview',
						'cbxtakeatour' ) ?></h3>
                <a href="#"
                   class="cbxtakeatour-btn <?php echo esc_attr( $button_class ); ?> cbxtakeatour-btn-<?php echo intval( $post_id ); ?>"><?php echo esc_attr( $tour_button_text,
						"cbxtakeatour" ) ?></a>
            </div>

            <div class="cbxtakeatour_box">
                <h3><?php esc_html_e( 'Tour Box Preview',
						'cbxtakeatour' ) ?></h3>
                <div class="cbxtakeatour_popover <?php echo esc_attr( $layout_class ); ?> cbxtakeatour_popover_<?php echo intval( $post_id ); ?> tour-tour tour-tour-0 fade show bs-cbxtakeatour_popover-top"
                     role="tooltip" id="step-0" x-placement="top"
                     style="position: relative;">
                    <div class="cbxtatarrow" style="left: 133px;"></div>
                    <h3 style="margin-top: 0px" class="cbxtakeatour_popover-header ">Welcome to CBX Tour!</h3>
                    <div class="cbxtakeatour_popover-body">Introduce new users to your product by walking them through
                        it step by step
                    </div>
                    <div class="cbxtakeatour_popover-navigation">
                        <div class="btn-group">
                            <a href="#" class="btn btn-sm btn-secondary disabled" disabled=""
                               tabindex="-1">« Prev </a> <a href="#" class="btn btn-sm btn-secondary">Next »</a>
                        </div>
                        <a href="#" style="float: right;" class="btn btn-sm btn-secondary">End tour</a>
                    </div>
                </div>
            </div>
        </div>
	<?php }// End cbxtourmetabox_preview_display


	/**
	 * Determines whether or not the current user has the ability to save meta data associated with this post.
	 *
	 * @param int $post_id The ID of the post being save
	 * @param bool                Whether or not the user has the ability to save this post.
	 */
	public function save_post( $post_id, $post ) {

		$post_type = 'cbxtour';

		// If this isn't a 'book' post, don't update it.
		if ( $post_type != $post->post_type ) {
			return;
		}

		if ( ! empty( $_POST['cbxtourmeta'] ) ) {
			$postData     = wp_unslash( $_POST['cbxtourmeta'] );
			$saveableData = array();

			$post_steps = isset( $postData['steps'] ) ? $postData['steps'] : array();
			if ( $this->user_can_save( $post_id, 'cbxtourmetabox', $postData['nonce'] ) ) {

				$i = 0;
				foreach ( $post_steps as $key => $deals ) {

					if ( is_numeric( $key ) ) {

						$saveableData['steps'][ $i ]['element']   = sanitize_text_field( $deals['element'] );
						$saveableData['steps'][ $i ]['content']   = wp_kses( $deals['content'],
							CBXTakeaTourHelper::allowedHtmlTags() );
						$saveableData['steps'][ $i ]['title']     = sanitize_text_field( $deals['title'] );
						$saveableData['steps'][ $i ]['placement'] = sanitize_text_field( $deals['placement'] );
						$saveableData['steps'][ $i ]['backdrop']  = intval( $deals['backdrop'] );

						$i ++;
					}
				}

				$saveableData['redirect_url'] = ( isset( $postData['redirect_url'] ) && $postData['redirect_url'] != '' ) ? esc_url( $postData['redirect_url'] ) : '';


				$saveableData['display']           = isset( $postData['display'] ) ? intval( $postData['display'] ) : 1;
				$saveableData['auto_start']        = isset( $postData['auto_start'] ) ? intval( $postData['auto_start'] ) : 0;
				$saveableData['tour_button_block'] = isset( $postData['tour_button_block'] ) ? intval( $postData['tour_button_block'] ) : 0;
				$saveableData['tour_button_align'] = isset( $postData['tour_button_align'] ) ? sanitize_text_field( $postData['tour_button_align'] ) : 'none';
				$saveableData['tour_button_text']  = isset( $postData['tour_button_text'] ) ? sanitize_text_field( $postData['tour_button_text'] ) : esc_html__( 'Take a Tour',
					'cbxtakeatour' );
				$saveableData['layout']            = sanitize_text_field( $postData['layout'] );

				$saveableData = apply_filters( 'cbxtakeatour_meta_saveabledata',
					$saveableData,
					$post_id,
					$postData );

				//endif;
				update_post_meta( $post_id, '_cbxtourmeta', $saveableData );


			}
		}
	}// End method save_post

	/**
	 * @param $post_id
	 * @param $action
	 * @param $nonce
	 *
	 * @return bool
	 */
	public function user_can_save( $post_id, $action, $nonce ) {

		$is_autosave    = wp_is_post_autosave( $post_id );
		$is_revision    = wp_is_post_revision( $post_id );
		$is_valid_nonce = ( isset( $nonce ) && wp_verify_nonce( $nonce, $action ) );

		// Return true if the user is able to save; otherwise, false.
		return ! ( $is_autosave || $is_revision ) && $is_valid_nonce;

	}// End user_can_save

	/**
	 * If we need to do something in upgrader process is completed for poll plugin
	 *
	 * @param $upgrader_object
	 * @param $options
	 */
	public function plugin_upgrader_process_complete( $upgrader_object, $options ) {
		if ( $options['action'] == 'update' && $options['type'] == 'plugin' ) {
			foreach ( $options['plugins'] as $each_plugin ) {
				if ( $each_plugin == CBXTAKEATOUR_BASE_NAME ) {
					set_transient( 'cbxtakeatour_upgraded_notice', 1 );
					break;
				}
			}
		}

	}//end plugin_upgrader_process_complete

	/**
	 * Show a notice to anyone who has just installed the plugin for the first time
	 * This notice shouldn't display to anyone who has just updated this plugin
	 */
	public function plugin_activate_upgrade_notices() {
		// Check the transient to see if we've just activated the plugin
		if ( get_transient( 'cbxtakeatour_activated_notice' ) ) {
			echo '<div class="notice notice-success is-dismissible" style="border-color: #6648fe !important;">';
			echo '<p>' . sprintf( __( 'Thanks for installing/deactivating <strong>CBX Tour - User Walkthroughs & Guided Tours</strong> V%s - Codeboxr Team',
					'cbxtakeatour' ),
					CBXTAKEATOUR_PLUGIN_VERSION ) . '</p>';
			echo '<p>' . sprintf( __( 'Check <a href="%s" target="_blank">Documentation</a> | Create <a class="button button-primary" style="background-color: #6648fe; border: 1px solid #6648fe; box-shadow: 0 2px 4px rgba(0, 5, 62, 0.3);" href="%s" target="_blank">Tour</a>',
					'cbxtakeatour' ),
					'https://codeboxr.com/product/cbx-tour-user-walkthroughs-guided-tours-for-wordpress/',
					admin_url( 'post-new.php?post_type=cbxtour' ) ) . '</p>';
			echo '</div>';

			// Delete the transient so we don't keep displaying the activation message
			delete_transient( 'cbxtakeatour_activated_notice' );

			$this->pro_addon_compatibility_campaign();
		}

		// Check the transient to see if we've just activated the plugin
		if ( get_transient( 'cbxtakeatour_upgraded_notice' ) ) {
			echo '<div class="notice notice-success is-dismissible" style="border-color: #6648fe !important;">';
			echo '<p>' . sprintf( __( 'Thanks for upgrading <strong>CBX Tour - User Walkthroughs & Guided Tours</strong> V%s , enjoy the new features and bug fixes - Codeboxr Team',
					'cbxtakeatour' ),
					CBXTAKEATOUR_PLUGIN_VERSION ) . '</p>';
			echo '<p>' . sprintf( __( 'Check <a href="%s" target="_blank">Documentation</a> | Create <a class="button button-primary" style="background-color: #6648fe; border: 1px solid #6648fe; box-shadow: 0 2px 4px rgba(0, 5, 62, 0.3);" href="%s" target="_blank">Tour</a>', 'cbxtakeatour' ),
					'https://codeboxr.com/product/cbx-tour-user-walkthroughs-guided-tours-for-wordpress/',
					admin_url( 'post-new.php?post_type=cbxtour' ) ) . '</p>';
			echo '</div>';

			// Delete the transient so we don't keep displaying the activation message
			delete_transient( 'cbxtakeatour_upgraded_notice' );

			$this->pro_addon_compatibility_campaign();
		}
	}//end plugin_activate_upgrade_notices

	/**
	 * Check plugin compatibility and pro addon install campaign
	 */
	public function pro_addon_compatibility_campaign() {
		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		//if the pro addon is active or installed
		if ( in_array( 'cbxtakeatourpro/cbxtakeatourpro.php',
				apply_filters( 'active_plugins',
					get_option( 'active_plugins' ) ) ) || defined( 'CBXTAKEATOURPRO_PLUGIN_NAME' ) ) {
			//plugin is activated

			$plugin_version = CBXTAKEATOURPRO_PLUGIN_VERSION;

			/*if(version_compare($plugin_version,'1.0.11', '<=') ){
				echo '<div class="notice notice-success is-dismissible"><p>' . esc_html__( 'CBX Petition Pro Addon Vx.x.x or any previous version is not compatible with CBX Petition Vx.x.x or later. Please update CBX Petition Pro Addon to version x.x.0 or later  - Codeboxr Team', 'cbxtakeatour' ) . '</p></div>';
			}*/
		} else {
			echo '<div class="notice notice-success is-dismissible" style="border-color: #6648fe !important;"><p>' . sprintf( __( 'CBX Tour - User Walkthroughs & Guided Tours Pro has extended features and more controls, <a target="_blank" href="%s">try it</a>  - Codeboxr Team',
					'cbxtakeatour' ),
					'https://codeboxr.com/product/cbx-tour-user-walkthroughs-guided-tours-for-wordpress//' ) . '</p></div>';
		}

	}//end pro_addon_compatibility_campaign


	/**
	 * Maximum one post for 'cbxtour' control
	 */
	public function remove_menus() {
		$button_count = wp_count_posts( 'cbxtour' );
		//remove add button option if already one button is created
		if ( $button_count->publish > 0 ) {
			do_action( 'cbxtour_remove', $this );

		}
	}//end remove_menus

	/**
	 * Create create new if have one tour
	 */
	public function cbxtour_remove_core() {
		remove_submenu_page( 'edit.php?post_type=cbxtour',
			'post-new.php?post_type=cbxtour' );        //remove add tour menu

		$result    = stripos( $_SERVER['REQUEST_URI'], 'post-new.php' );
		$post_type = isset( $_REQUEST['post_type'] ) ? esc_attr( wp_unslash( $_REQUEST['post_type'] ) ) : '';

		if ( $result !== false ) {
			if ( $post_type == 'cbxtour' ) {
				wp_redirect( get_option( 'siteurl' ) . '/wp-admin/edit.php?post_type=cbxtour&cbxtour_error=true' );
			}

		}
	}//end cbxtour_remove_core


	/**
	 * Admin notice if user try to create new button in free version
	 */
	function cbxtour_notice() {
		if ( isset( $_GET['cbxtour_error'] ) ) {
			add_action( 'admin_notices', array( $this, 'permissions_admin_notice' ) );
		}
	}//end cbxtour_notice

	/**
	 * Showing Admin notice
	 *
	 */
	function permissions_admin_notice() {
		echo "<div id='permissions-warning' class='error fade'><p><strong>" . sprintf( __( 'Sorry, you can not create more than one tour in free version, <a target="_blank" href="%s">Grab Pro</a>',
				'cbxtakeatour' ),
				'https://codeboxr.com/product/cbx-tour-user-walkthroughs-guided-tours-for-wordpress//' ) . "</strong></p></div>";
	}//end permissions_admin_notice


	/**
	 * Init all gutenberg blocks
	 */
	public function gutenberg_blocks() {
		if ( ! function_exists( 'register_block_type' ) ) {
			// Gutenberg is not active.
			return;
		}

		$active_tours = array();

		$active_tours[] = array(
			'label' => esc_html__( 'Select Tour', 'cbxtakeatour' ),
			'value' => '0',
		);

		$args = array(
			'post_type'      => 'cbxtour',
			'orderby'        => 'ID',
			'order'          => 'DESC',
			'post_status'    => 'publish',
			'posts_per_page' => - 1,
		);

		$tour_posts = get_posts( $args );

		if ( ! is_admin() ) {
			global $post;
		}

		foreach ( $tour_posts as $post ) :
			CBXTakeaTourHelper::setup_admin_postdata( $post );

			$post_id    = intval( get_the_ID() );
			$post_title = get_the_title();

			$active_tours[] = array(
				'label' => esc_html( $post_title ),
				'value' => intval( $post_id ),
			);
		endforeach;
		CBXTakeaTourHelper::wp_reset_admin_postdata();

		$align_arr = array(
			''       => esc_html__( 'Select from tour post meta setting', 'cbxtakeatour' ),
			'center' => esc_html__( 'Center', 'cbxtakeatour' ),
			'left'   => esc_html__( 'Left', 'cbxtakeatour' ),
			'right'  => esc_html__( 'Right', 'cbxtakeatour' ),
			'none'   => esc_html__( 'None', 'cbxtakeatour' ),
		);

		$align_options = array();
		foreach ( $align_arr as $key => $value ) {
			$align_options[] = array(
				'label' => esc_attr( $value ),
				'value' => esc_attr( $key ),
			);
		}

		$yes_no_arr = array(
			''       => esc_html__( 'Select from tour post meta setting', 'cbxtakeatour' ),
			'1' => esc_html__( 'Yes', 'cbxtakeatour' ),
			'0'   => esc_html__( 'No', 'cbxtakeatour' ),
		);

		$yes_no_options = array();
		foreach ( $yes_no_arr as $key => $value ) {
			$yes_no_options[] = array(
				'label' => esc_attr( $value ),
				'value' => esc_attr( $key ),
			);
		}

		wp_register_style( 'cbxtakeatour-block', plugin_dir_url( __FILE__ ) . '../assets/css/cbxtakeatour-block.css', array(), filemtime( plugin_dir_path( __FILE__ ) . '../assets/css/cbxtakeatour-block.css' ) );

		wp_register_script( 'cbxtakeatour-block',
			plugin_dir_url( __FILE__ ) . '../assets/js/cbxtakeatour-block.js',
			array(
				'wp-blocks',
				'wp-element',
				'wp-components',
				'wp-editor',
				'jquery',
			),
			filemtime( plugin_dir_path( __FILE__ ) . '../assets/js/cbxtakeatour-block.js' ) );


		$js_vars = apply_filters( 'cbxtakeatour_block_js_vars',
			array(
				'block_title'      => esc_html__( 'CBX Tour - User Walkthroughs', 'cbxtakeatour' ),
				'block_category'   => 'codeboxr',
				'block_icon'       => 'universal-access-alt',
				'general_settings' => array(
					'title'         => esc_html__( 'CBX Tour - User Walkthroughs Settings', 'cbxtakeatour' ),
					'id'            => esc_html__( 'Select Tour', 'cbxtakeatour' ),
					'id_options'    => $active_tours,
					'button_text'   => esc_html__( 'Button Text (Leave empty to use tour post meta)', 'cbxtakeatour' ),
					'display'       => esc_html__( 'Display Tour Button', 'cbxtakeatour' ),
					'auto_start'    => esc_html__( 'Tour Auto-start', 'cbxtakeatour' ),
					'block'         => esc_html__( 'Block Button (Full Width)', 'cbxtakeatour' ),
					'align'         => esc_html__( 'Button Align', 'cbxtakeatour' ),
					'align_options' => $align_options,
					'yes_no_options' => $yes_no_options,
				),
			) );

		wp_localize_script( 'cbxtakeatour-block', 'cbxtakeatour_block', $js_vars );

		register_block_type( 'codeboxr/cbxtakeatour',
			array(
				'editor_script'   => 'cbxtakeatour-block',
				//'editor_style'    => 'cbxtakeatour-block',
				'attributes'      => apply_filters( 'cbxtakeatour_block_attributes',
					array(
						'id'          => array(
							'type'    => 'integer',
							'default' => 0,
						),
						'button_text' => array(
							'type'    => 'string',
							'default' => '',
						),
						'display'     => array(
							'type'    => 'string',
							'default' => '',
						),
						'auto_start'  => array(
							'type'    => 'string',
							'default' => '',
						),
						'block'       => array(
							'type'    => 'string',
							'default' => '',
						),
						'align'       => array(
							'type'    => 'string',
							'default' => '',
						),
					) ),
				'render_callback' => array( $this, 'cbxtakeatour_block_render' ),
			) );

	}//end method gutenberg_blocks

	/**
	 * gutenberg server side render
	 *
	 * @param $attr
	 *
	 * @return string
	 */
	public function cbxtakeatour_block_render( $attr ) {
		$arr = array();

		$arr['id'] = isset( $attr['id'] ) ? intval( $attr['id'] ) : 0;

		if ( intval( $arr['id'] ) == 0 ) {
			return esc_html__( 'Please select  a tour', 'cbxtakeatour' );
		}

		$arr['button_text'] = isset( $attr['button_text'] ) ? esc_attr( wp_unslash( $attr['button_text'] )) : '';
		$arr['display']    = isset( $attr['display'] ) ? esc_attr( wp_unslash( $attr['display'] ) ) : '';
		$arr['auto_start'] = isset( $attr['auto_start'] ) ? esc_attr( wp_unslash( $attr['auto_start'] ) ) : '';
		$arr['block']      = isset( $attr['block'] ) ? esc_attr( wp_unslash( $attr['block'] ) ) : '';
		$arr['align']      = isset( $attr['align'] ) ? esc_attr( wp_unslash( $attr['align'] ) ) : '';


		/*$arr['display']    = ( $arr['display'] == 'true' ) ? 1 : 0;
		$arr['auto_start'] = ( $arr['auto_start'] == 'true' ) ? 1 : 0;
		$arr['block']      = ( $arr['block'] == 'true' ) ? 1 : 0;*/

		$arr = apply_filters( 'cbxtakeatour_shortcode_builder_block_attr', $arr, $attr );

		$attr_html = '';

		foreach ( $arr as $key => $value ) {
			$attr_html .= ' ' . $key . '="' . $value . '" ';
		}

		return do_shortcode( '[cbxtakeatour ' . $attr_html . ']' );
		//return '[cbxtakeatour ' . $attr_html . ']';

	}//end method cbxtakeatour_block_render

	/**
	 * Register New Gutenberg block Category if need
	 *
	 * @param $categories
	 * @param $post
	 *
	 * @return mixed
	 */
	public function gutenberg_block_categories( $categories, $post ) {
		$found = false;
		foreach ( $categories as $category ) {
			if ( $category['slug'] == 'codeboxr' ) {
				$found = true;
				break;
			}
		}

		if ( ! $found ) {
			return array_merge(
				$categories,
				array(
					array(
						'slug'  => 'codeboxr',
						'title' => esc_html__( 'CBX Blocks', 'cbxtakeatour' ),
					),
				)
			);
		}

		return $categories;
	}//end method gutenberg_block_categories


	/**
	 * Enqueue style for block editor
	 */
	public function enqueue_block_editor_assets() {
		wp_register_style( 'bootstrap-tour-standalone',
			plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone.css',
			array(),
			$this->version,
			'all' );
		wp_register_style( 'bootstrap-tour-standalone-red',
			plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone_red.css',
			array(),
			$this->version,
			'all' );

		wp_register_style( 'cbxtakeatour-public',
			plugin_dir_url( __FILE__ ) . '../assets/css/cbxtakeatour-public.css',
			array( 'bootstrap-tour-standalone' ),
			$this->version,
			'all' );

		wp_enqueue_style( 'bootstrap-tour-standalone' );
		wp_enqueue_style( 'cbxtakeatour-public' );

	}//end method enqueue_block_editor_assets

	/**
	 * Show action links on the plugin screen.
	 *
	 * @param mixed $links Plugin Action links.
	 *
	 * @return  array
	 */
	public function plugin_action_links( $links ) {
		$action_links = array(
			'settings' => '<a style="color:#6648fe !important; font-weight: bold;" href="' . admin_url( 'edit.php?post_type=cbxtour' ) . '" aria-label="' . esc_attr__( 'Explore Tours', 'cbxtakeatour' ) . '">' . esc_html__( 'Explore Tours', 'cbxtakeatour' ) . '</a>',
		);

		return array_merge( $action_links, $links );
	}//end plugin_action_links

	/**
	 * Filters the array of row meta for each/specific plugin in the Plugins list table.
	 * Appends additional links below each/specific plugin on the plugins page.
	 *
	 * @access  public
	 *
	 * @param array $links_array An array of the plugin's metadata
	 * @param string $plugin_file_name Path to the plugin file
	 * @param array $plugin_data An array of plugin data
	 * @param string $status Status of the plugin
	 *
	 * @return  array       $links_array
	 */
	public function plugin_row_meta( $links_array, $plugin_file_name, $plugin_data, $status ) {
		if ( strpos( $plugin_file_name, CBXTAKEATOUR_BASE_NAME ) !== false ) {
			if ( ! function_exists( 'is_plugin_active' ) ) {
				include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			}

			$links_array[] = '<a target="_blank" style="color:#6648fe !important; font-weight: bold;" href="https://wordpress.org/support/plugin/cbxtakeatour/" aria-label="' . esc_attr__( 'Free Support', 'cbxtakeatour' ) . '">' . esc_html__( 'Free Support', 'cbxtakeatour' ) . '</a>';

			$links_array[] = '<a target="_blank" style="color:#6648fe !important; font-weight: bold;" href="https://wordpress.org/plugins/cbxtakeatour/#reviews" aria-label="' . esc_attr__( 'Reviews', 'cbxtakeatour' ) . '">' . esc_html__( 'Reviews', 'cbxtakeatour' ) . '</a>';

			$links_array[] = '<a target="_blank" style="color:#6648fe !important; font-weight: bold;" href="https://codeboxr.com/documentation-for-cbx-tour-user-walkthroughs-guided-tours-for-wordpress/" aria-label="' . esc_attr__( 'Documentation', 'cbxtakeatour' ) . '">' . esc_html__( 'Documentation', 'cbxtakeatour' ) . '</a>';


			if ( in_array( 'cbxtakeatourpro/cbxtakeatourpro.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) || defined( 'CBXTAKEATOURPRO_PLUGIN_NAME' ) ) {
				$links_array[] = '<a target="_blank" style="color:#6648fe !important; font-weight: bold;" href="https://codeboxr.com/contact-us/" aria-label="' . esc_attr__( 'Pro Support', 'cbxtakeatour' ) . '">' . esc_html__( 'Pro Support', 'cbxtakeatour' ) . '</a>';
			} else {
				$links_array[] = '<a target="_blank" style="color:#6648fe !important; font-weight: bold;" href="https://codeboxr.com/product/cbx-tour-user-walkthroughs-guided-tours-for-wordpress" aria-label="' . esc_attr__( 'Try Pro Addon', 'cbxtakeatour' ) . '">' . esc_html__( 'Try Pro Addon', 'cbxtakeatour' ) . '</a>';
			}
		}

		return $links_array;
	}//end plugin_row_meta

	/**
	 * Reset plugin data
	 */
	public function plugin_reset() {


		if ( isset( $_REQUEST['page'] ) && $_REQUEST['page'] == 'cbxtakeatoursettings' && isset( $_REQUEST['cbxtakeatour_fullreset'] ) && intval( $_REQUEST['cbxtakeatour_fullreset'] ) == 1 ) {
			global $wpdb;



			$option_prefix = 'cbxtakeatour_';

			//delete options
			$option_values = CBXTakeaTourHelper::getAllOptionNames();

			foreach ( $option_values as $key => $accounting_option_value ) {
				delete_option( $accounting_option_value['option_name'] );
			}

			do_action( 'cbxtakeatour_plugin_option_delete' );


			//delete tables
			$table_names  = CBXTakeaTourHelper::getAllDBTablesList();
			$sql          = "DROP TABLE IF EXISTS " . implode( ', ', array_values( $table_names ) );
			$query_result = $wpdb->query( $sql );

			do_action( 'cbxtakeatour_plugin_table_delete' );

			// create plugin's core table tables
			//CBXUseronlineHelper::create_tables();

			//please note that, the default options will be created by default

			//3rd party plugin's table creation
			do_action( 'cbxtakeatour_plugin_reset');


			$this->settings_api->set_sections( $this->get_settings_sections() );
			$this->settings_api->set_fields( $this->get_settings_fields() );
			$this->settings_api->admin_init();

			wp_safe_redirect( admin_url( 'edit.php?post_type=cbxtour&page=cbxtakeatoursettings' ) );
			exit();
		}
	}//end method plugin_reset

	/**
	 * Add our self-hosted autoupdate plugin to the filter transient
	 *
	 * @param $transient
	 *
	 * @return object $ transient
	 */
	public function pre_set_site_transient_update_plugins_pro_addon($transient){
		// Extra check for 3rd plugins
		if ( isset( $transient->response[ 'cbxtakeatourpro/cbxtakeatourpro.php' ] ) ) {
			return $transient;
		}

		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugin_info = array();
		$all_plugins = get_plugins();
		if(!isset($all_plugins['cbxtakeatourpro/cbxtakeatourpro.php'])){
			return $transient;
		}
		else{
			$plugin_info = $all_plugins['cbxtakeatourpro/cbxtakeatourpro.php'];
		}

		$remote_version = '1.0.2';

		if ( version_compare( $plugin_info['Version'], $remote_version, '<' ) ) {
			$obj = new stdClass();
			$obj->slug = 'cbxtakeatourpro';
			$obj->new_version = $remote_version;
			$obj->plugin = 'cbxtakeatourpro/cbxtakeatourpro.php';
			$obj->url = '';
			$obj->package = false;
			$obj->name = 'CBX Tour - User Walkthroughs & Guided Tours Pro Addon';
			$transient->response[ 'cbxtakeatourpro/cbxtakeatourpro.php' ] = $obj;
		}

		return $transient;
	}//end pre_set_site_transient_update_plugins_pro_addons

	/**
	 * Pro Addon update message
	 */
	public function plugin_update_message_pro_addons(){
		echo ' '.sprintf(__('Check how to <a style="color:#9c27b0 !important; font-weight: bold;" href="%s"><strong>Update manually</strong></a> , download latest version from <a style="color:#9c27b0 !important; font-weight: bold;" href="%s"><strong>My Account</strong></a> section of Codeboxr.com', 'cbxtakeatourpro'), 'https://codeboxr.com/manual-update-pro-addon/', 'https://codeboxr.com/my-account/');
	}//end plugin_update_message_pro_addons

}//end class CBXTakeaTour_Admin