<?php
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Helper class with all static methods
 *
 * Class CBXTakeaTourHelper
 */
class CBXTakeaTourHelper {
	/**
	 * Custom helper function for take a tour button display and integration
	 *
	 * @param array $tour_data
	 *
	 * @return string
	 */
	public static function display_tour( $tour_data = array() ) {
		$id = isset( $tour_data['id'] ) ? intval( $tour_data['id'] ) : 0;


		$display = true;

		if(!apply_filters('cbxtakeatour_display_tour', $display, $tour_data, $id)) return;


		if ( $id > 0 && is_array( $tour_data ) && sizeof( $tour_data ) == 0 ) {
			$tour_data = get_post_meta( $id, '_cbxtourmeta', true );
		}

		if ( ! is_array( $tour_data ) || ( is_array( $tour_data ) && sizeof( $tour_data ) == 0 ) ) {
			return '';
		}

		$layout  = isset( $tour_data['layout'] ) ? esc_attr( $tour_data['layout'] ) : 'basic';
		$layouts = array_keys( CBXTakeaTourHelper::cbxtakeatour_layouts() );

		//if pro layout set and pro addon is disabled, let's fallback to 'basic'
		if ( ! in_array( $layout, $layouts ) ) {
			$layout = 'basic';
		}

		//button text
		$tour_button_text = ( isset( $tour_data['tour_button_text'] ) && ( $tour_data['tour_button_text'] != '' ) ) ? esc_attr( wp_unslash( $tour_data['tour_button_text'] ) ) : esc_html__( 'Take a Tour', 'cbxtakeatour' );

		//button align
		$tour_button_align = ( isset( $tour_data['tour_button_align'] ) && ( $tour_data['tour_button_align'] != '' ) ) ? esc_attr( $tour_data['tour_button_align'] ) : 'center';
		//for fail safe
		if ( ! in_array( $tour_button_align, array( 'left', 'center', 'right', 'none' ) ) ) {
			$tour_button_align = 'center';
		}

		//button block
		$tour_button_block = isset( $tour_data['tour_button_block'] ) ? intval( $tour_data['tour_button_block'] ) : 0;

		$button_block_class = ( intval( $tour_button_block ) ) ? 'cbxtakeatour-btn-block' : '';

		$button_class = 'cbxtakeatour-btn-' . $layout;

		$display    = isset( $tour_data['display'] ) ? $tour_data['display'] : 1;
		$auto_start = isset( $tour_data['auto_start'] ) ? $tour_data['auto_start'] : 0;

		$ready_layouts = CBXTakeaTourHelper::cbxtakeatour_layout_ready_styles();
		if ( array_key_exists( $layout, $ready_layouts ) ) {
			$custom_css = CBXTakeaTourHelper::add_custom_css( $id, CBXTakeaTourHelper::cbxtakeatour_layout_ready_style( $layout ) );
			wp_register_style( 'cbxtakeatour-public-inline', false, array( 'cbxtakeatour-public' ) );
			wp_enqueue_style( 'cbxtakeatour-public-inline' );
			wp_add_inline_style( 'cbxtakeatour-public-inline', $custom_css );
		}

		do_action( 'cbxtakeatour_display_tour_public_enqueue', $layout, $id, $tour_data );

		wp_add_inline_script( 'cbxtakeatour-public',
			' cbxtakeatour.steps[' . $id . ']=' . json_encode( $tour_data ) . '; ',
			'before' );

		$display_style = ( 0 == $display ) ? ' display: none; ' : '';
		$html          = '<span class="cbxtakeatour-btn-parent cbxtakeatour-btn-parent-' . esc_attr( $tour_button_align ) . '"><a title="'.get_the_title($id).'" style="' . $display_style . '" id="cbxtakeatour-' . $id . '" class="cbxtakeatour cbxtakeatour-btn ' . esc_attr( $button_class ) . ' cbxtakeatour-btn-' . $id . ' cbxtakeatour-btn-' . esc_attr( $tour_button_align ) . ' ' . esc_attr( $button_block_class ) . '"  data-auto_start="' . $auto_start . '" data-step_id="' . $id . '" href="#">' . esc_attr( $tour_button_text, 'cbxtakeatour' ) . '</a></span>';

		return $html;
	}//end display_tour

	/**
	 * Placement helper array
	 *
	 * @return mixed|void
	 */
	public static function cbxtakeatour_placement() {
		$placement_array = array(
			'top'    => esc_html__( 'Top', 'cbxtakeatour' ),
			'bottom' => esc_html__( 'Bottom', 'cbxtakeatour' ),
			'left'   => esc_html__( 'Left', 'cbxtakeatour' ),
			'right'  => esc_html__( 'Right', 'cbxtakeatour' ),
		);

		return apply_filters( 'cbxtakeatour_placement', $placement_array );
	}//end cbxtakeatour_placement


	/**
	 * Boolean yes no array helper
	 *
	 * @return mixed|void
	 */
	public static function cbxtakeatour_bool_arr() {
		$cbxtakeatour_bool_arr = array(
			1 => esc_html__( 'Yes', 'cbxtakeatour' ),
			0 => esc_html__( 'No', 'cbxtakeatour' ),
		);

		return apply_filters( 'cbxtakeatour_bool_arr', $cbxtakeatour_bool_arr );
	}//end cbxtakeatour_bool_arr

	/*/**
	 * @return mixed|void
	 */
	public static function cbxtakeatour_display() {
		$display_array = array(
			'1' => esc_html__( 'Show', 'cbxtakeatour' ),
			'0' => esc_html__( 'Hide', 'cbxtakeatour' ),
		);

		return apply_filters( 'cbxtakeatour_display', $display_array );
	}//end cbxtakeatour_display*/

	/**
	 * List style layout types
	 *
	 * @return mixed|void
	 */
	public static function cbxtakeatour_layouts() {
		$layout_options = array(
			'basic' => esc_html__( 'Default', 'cbxtakeatour' ),
			'red'   => esc_html__( 'Red', 'cbxtakeatour' ),
			'blue'  => esc_html__( 'Blue', 'cbxtakeatour' ),
			'green' => esc_html__( 'Green', 'cbxtakeatour' )
		);

		return apply_filters( 'cbxtakeatour_layouts', $layout_options );
	}//end cbxtakeatour_layouts

	/**
	 * Ready layout styles
	 *
	 * @return mixed|void
	 */
	public static function cbxtakeatour_layout_ready_styles() {
		$layout_ready_styles = array(
			'red'   => array(
				'body_bg_color'               => '#ffebee',
				'body_text_color'             => '#212529',
				'body_border_color'           => 'rgba(244, 67, 54, 0.3)',
				'heading_bg_color'            => '#f44336',
				'heading_text_color'          => '#ffffff',
				'heading_border_bottom_color' => '#d32f2f',

				'button_color'                       => '#e53935',
				'button_text_color'                  => '#ffffff',
				'button_border_color'                => 'rgba(211, 47, 47, 0.3)',
				'button_hover_color'                 => '#ef5350',
				'button_hover_border_color'          => 'rgba(229, 57, 53, 0.3)',
				'button_disabled_hover_border_color' => 'rgba(211, 47, 47, 0.3)',
				'button_active_color'                => '#ef5350',
				'button_active_border_color'         => '#e53935',
				'button_focus_color'                 => 'rgba(244, 67, 54, 0.3)',
				'arrow_color'                        => '#ffebee',
				'arrow_border_color'                 => 'rgba(244, 67, 54, 0.3)',

				'tour_button_color'               => '#f44336',
				'tour_button_text_color'          => '#ffffff',
				'tour_button_border_color'        => '#f44336',
				'tour_button_hover_color'         => '#ef5350',
				'tour_button_hover_border_color'  => '#ef5350',
				'tour_button_active_color'        => '#ef5350',
				'tour_button_active_border_color' => '#e53935',
				'tour_button_focus_color'         => 'rgba(244, 67, 54, 0.3)',
			),
			'green' => array(
				'body_bg_color'               => '#e8f5e9',
				'body_text_color'             => '#212529',
				'body_border_color'           => 'rgba(76, 175, 80, 0.3)',
				'heading_bg_color'            => '#4caf50',
				'heading_text_color'          => '#ffffff',
				'heading_border_bottom_color' => '#388e3c',

				'button_color'                       => '#43a047',
				'button_text_color'                  => '#ffffff',
				'button_border_color'                => 'rgba(56, 142, 60, 0.3)',
				'button_hover_color'                 => '#66bb6a',
				'button_hover_border_color'          => 'rgba(67, 160, 71, 0.3)',
				'button_disabled_hover_border_color' => 'rgba(56, 142, 60, 0.3)',
				'button_active_color'                => '#66bb6a',
				'button_active_border_color'         => '#43a047',
				'button_focus_color'                 => 'rgba(76, 175, 80, 0.3)',
				'arrow_color'                        => '#e8f5e9',
				'arrow_border_color'                 => 'rgba(76, 175, 80, 0.3)',

				'tour_button_color'               => '#4caf50',
				'tour_button_text_color'          => '#ffffff',
				'tour_button_border_color'        => '#4caf50',
				'tour_button_hover_color'         => '#66bb6a',
				'tour_button_hover_border_color'  => '#66bb6a',
				'tour_button_active_color'        => '#66bb6a',
				'tour_button_active_border_color' => '#43a047',
				'tour_button_focus_color'         => 'rgba(76, 175, 80, 0.3)',
			),
			'blue'  => array(
				'body_bg_color'               => '#e3f2fd',
				'body_text_color'             => '#212529',
				'body_border_color'           => 'rgba(33, 150, 243, 0.3)',
				'heading_bg_color'            => '#2196f3',
				'heading_text_color'          => '#ffffff',
				'heading_border_bottom_color' => '#1976d2',

				'button_color'                       => '#1e88e5',
				'button_text_color'                  => '#ffffff',
				'button_border_color'                => 'rgba(25, 118, 210, 0.3)',
				'button_hover_color'                 => '#42a5f5',
				'button_hover_border_color'          => 'rgba(30, 136, 229, 0.3)',
				'button_disabled_hover_border_color' => 'rgba(25, 118, 210, 0.3)',
				'button_active_color'                => '#42a5f5',
				'button_active_border_color'         => '#1e88e5',
				'button_focus_color'                 => 'rgba(33, 150, 243, 0.3)',
				'arrow_color'                        => '#e3f2fd',
				'arrow_border_color'                 => 'rgba(33, 150, 243, 0.3)',

				'tour_button_color'               => '#2196f3',
				'tour_button_text_color'          => '#ffffff',
				'tour_button_border_color'        => '#2196f3',
				'tour_button_hover_color'         => '#42a5f5',
				'tour_button_hover_border_color'  => '#42a5f5',
				'tour_button_active_color'        => '#42a5f5',
				'tour_button_active_border_color' => '#1e88e5',
				'tour_button_focus_color'         => 'rgba(33, 150, 243, 0.3)',
			)
		);

		return apply_filters( 'cbxtakeatour_layout_ready_styles', $layout_ready_styles );
	}//end cbxtakeatour_layout_ready_styles

	/**
	 * Get a ready layout style
	 *
	 * @param string $layout
	 *
	 * @return array|mixed|string
	 */
	public static function cbxtakeatour_layout_ready_style( $layout = '' ) {
		if ( $layout == '' ) {
			return '';
		}

		$layout = esc_attr( $layout );

		$layout_ready_styles = CBXTakeaTourHelper::cbxtakeatour_layout_ready_styles();

		$layout_ready_style = isset( $layout_ready_styles[ $layout ] ) ? $layout_ready_styles[ $layout ] : array();

		return apply_filters( 'cbxtakeatour_layout_ready_style', $layout_ready_style );
	}//end cbxtakeatour_layout_ready_style

	/**
	 * HTML elements, attributes, and attribute values will occur in your output
	 * @return array
	 */
	public static function allowedHtmlTags() {
		$allowed_html_tags = array(
			'a'      => array(
				'href'  => array(),
				'title' => array(),
				//'class' => array(),
				//'data'  => array(),
				//'rel'   => array(),
			),
			'br'     => array(),
			'em'     => array(),
			'ul'     => array(//'class' => array(),
			),
			'ol'     => array(//'class' => array(),
			),
			'li'     => array(//'class' => array(),
			),
			'strong' => array(),
			'p'      => array(
				//'class' => array(),
				//'data'  => array(),
				//'style' => array(),
			),
			'span'   => array(
				//					'class' => array(),
				//'style' => array(),
			),
		);

		return apply_filters( 'cbxtakeatour_allowed_html_tags', $allowed_html_tags );
	}//end allowedHtmlTags

	/**
	 * Add custom style for any step
	 *
	 * @param int $id
	 *
	 * @return string
	 */
	public static function add_custom_css( $id = 0, $meta = array() ) {
		$id = intval( $id );
		if ( $id == 0 ) {
			return '';
		}

		if ( ! is_array( $meta ) || ( is_array( $meta ) && sizeof( $meta ) == 0 ) ) {
			return '';
		}

		//tour button
		$tour_button_text_size    = ( isset( $meta['tour_button_text_size'] ) && ( $meta['tour_button_text_size'] != '' ) ) ? $meta['tour_button_text_size'] : 14;
		$padding_top              = ( isset( $meta['padding']['pt'] ) && ( $meta['padding']['pt'] != '' ) ) ? $meta['padding']['pt'] : 8;
		$padding_right            = ( isset( $meta['padding']['pr'] ) && ( $meta['padding']['pr'] != '' ) ) ? $meta['padding']['pr'] : 12;
		$padding_bottom           = ( isset( $meta['padding']['pb'] ) && ( $meta['padding']['pb'] != '' ) ) ? $meta['padding']['pb'] : 8;
		$padding_left             = ( isset( $meta['padding']['pl'] ) && ( $meta['padding']['pl'] != '' ) ) ? $meta['padding']['pl'] : 12;
		$tour_button_color        = ( isset( $meta['tour_button_color'] ) && ( $meta['tour_button_color'] != '' ) ) ? $meta['tour_button_color'] : '#868e96';
		$tour_button_text_color   = ( isset( $meta['tour_button_text_color'] ) && ( $meta['tour_button_text_color'] != '' ) ) ? $meta['tour_button_text_color'] : '#ffffff';
		$tour_button_border_color = ( isset( $meta['tour_button_border_color'] ) && ( $meta['tour_button_border_color'] != '' ) ) ? $meta['tour_button_border_color'] : '#868e96';

		$tour_button_hover_color        = ( isset( $meta['tour_button_hover_color'] ) && ( $meta['tour_button_hover_color'] != '' ) ) ? $meta['tour_button_hover_color'] : '#727b84';
		$tour_button_hover_text_color   = ( isset( $meta['tour_button_hover_text_color'] ) && ( $meta['tour_button_hover_text_color'] != '' ) ) ? $meta['tour_button_hover_text_color'] : '#ffffff';
		$tour_button_hover_border_color = ( isset( $meta['tour_button_hover_border_color'] ) && ( $meta['tour_button_hover_border_color'] != '' ) ) ? $meta['tour_button_hover_border_color'] : '#6c757d';

		$tour_button_active_color        = ( isset( $meta['tour_button_active_color'] ) && ( $meta['tour_button_active_color'] != '' ) ) ? $meta['tour_button_active_color'] : '#727b84';
		$tour_button_active_border_color = ( isset( $meta['tour_button_active_border_color'] ) && ( $meta['tour_button_active_border_color'] != '' ) ) ? $meta['tour_button_active_border_color'] : '#6c757d';

		$tour_button_focus_color = ( isset( $meta['tour_button_focus_color'] ) && ( $meta['tour_button_focus_color'] != '' ) ) ? $meta['tour_button_focus_color'] : 'rgba(134, 142, 150, 0.5)';
		//end tour button

		//get meta for custom style for this $step id
		$body_bg_color     = ( isset( $meta['body_bg_color'] ) && ( $meta['body_bg_color'] != '' ) ) ? $meta['body_bg_color'] : '#ffffff';
		$body_text_color   = ( isset( $meta['body_text_color'] ) && ( $meta['body_text_color'] != '' ) ) ? $meta['body_text_color'] : '#212529';
		$body_border_color = ( isset( $meta['body_border_color'] ) && ( $meta['body_border_color'] != '' ) ) ? $meta['body_border_color'] : 'rgba(0, 0, 0, 0.25)';

		$heading_bg_color            = ( isset( $meta['heading_bg_color'] ) && ( $meta['heading_bg_color'] != '' ) ) ? $meta['heading_bg_color'] : '#f7f7f7';
		$heading_text_color          = ( isset( $meta['heading_text_color'] ) && ( $meta['heading_text_color'] != '' ) ) ? $meta['heading_text_color'] : '#444444';
		$heading_border_bottom_color = ( isset( $meta['heading_border_bottom_color'] ) && ( $meta['heading_border_bottom_color'] != '' ) ) ? $meta['heading_border_bottom_color'] : '#ebebeb';


		/*Tour Box color*/
		$button_color        = ( isset( $meta['button_color'] ) && ( $meta['button_color'] != '' ) ) ? $meta['button_color'] : '#868e96';
		$button_text_color   = ( isset( $meta['button_text_color'] ) && ( $meta['button_text_color'] != '' ) ) ? $meta['button_text_color'] : '#ffffff';
		$button_border_color = ( isset( $meta['button_border_color'] ) && ( $meta['button_border_color'] != '' ) ) ? $meta['button_border_color'] : 'rgba(104, 104, 104, 0.3)';

		$button_hover_color        = ( isset( $meta['button_hover_color'] ) && ( $meta['button_hover_color'] != '' ) ) ? $meta['button_hover_color'] : '#727b84';
		$button_hover_text_color   = ( isset( $meta['button_hover_text_color'] ) && ( $meta['button_hover_text_color'] != '' ) ) ? $meta['button_hover_text_color'] : '#ffffff';
		$button_hover_border_color = ( isset( $meta['button_hover_border_color'] ) && ( $meta['button_hover_border_color'] != '' ) ) ? $meta['button_hover_border_color'] : 'rgba(104, 104, 104, 0.3)';

		$button_disabled_hover_border_color = ( isset( $meta['button_disabled_hover_border_color'] ) && ( $meta['button_disabled_hover_border_color'] != '' ) ) ? $meta['button_disabled_hover_border_color'] : 'rgba(104, 104, 104, 0.3)';

		$button_active_color        = ( isset( $meta['button_active_color'] ) && ( $meta['button_active_color'] != '' ) ) ? $meta['button_active_color'] : '#727b84';
		$button_active_border_color = ( isset( $meta['button_active_border_color'] ) && ( $meta['button_active_border_color'] != '' ) ) ? $meta['button_active_border_color'] : '#6c757d';

		$button_focus_color = ( isset( $meta['button_focus_color'] ) && ( $meta['button_focus_color'] != '' ) ) ? $meta['button_focus_color'] : 'rgba(134, 142, 150, 0.5)';

		$arrow_color        = ( isset( $meta['arrow_color'] ) && ( $meta['arrow_color'] != '' ) ) ? $meta['arrow_color'] : '#ffffff';
		$arrow_border_color = ( isset( $meta['arrow_border_color'] ) && ( $meta['arrow_border_color'] != '' ) ) ? $meta['arrow_border_color'] : 'rgba(0, 0, 0, 0.3)';


		//button start
		$custom_css = '.cbxtakeatour-btn{
	padding: ' . $padding_top . 'px ' . $padding_right . 'px ' . $padding_bottom . 'px ' . $padding_left . 'px;
	font-size:' . $tour_button_text_size . 'px;
}

.cbxtakeatour-btn-' . $id . ' {
    color: ' . $tour_button_text_color . ';
    background-color: ' . $tour_button_color . ';
    border-color: ' . $tour_button_border_color . ';
}

.cbxtakeatour-btn-' . $id . ':hover {
    color: ' . $tour_button_hover_text_color . ';
    background-color: ' . $tour_button_hover_color . '!important;
    border-color: ' . $tour_button_hover_border_color . ';
}

.cbxtakeatour-btn-' . $id . ':focus, .cbxtakeatour-btn-' . $id . '.focus {
    box-shadow: 0 0 0 3px ' . $tour_button_focus_color . '!important;
}

.cbxtakeatour-btn-' . $id . ':active, .cbxtakeatour-btn-' . $id . '.active
 {
    background-color: ' . $tour_button_active_color . ';
    border-color: ' . $tour_button_active_border_color . ';
}';

		//button end

		//pop hover start
		$custom_css .= ' 
.cbxtakeatour_popover_' . $id . ' {
  background-color: ' . $body_bg_color . ';
  border: 1px solid ' . $body_border_color . ';
}

.cbxtakeatour_popover_' . $id . ' .cbxtakeatour_popover-header {
  color: ' . $heading_text_color . ';
  background-color:' . $heading_bg_color . ';
  border-bottom: 1px solid ' . $heading_border_bottom_color . ';
}
.cbxtakeatour_popover_' . $id . ' .cbxtakeatour_popover-body{
  color: ' . $body_text_color . ';
}

.cbxtakeatour_popover_' . $id . ' .btn-secondary {
  color: ' . $button_text_color . ';
  background-color: ' . $button_color . ';
  border-color: ' . $button_border_color . ';
}
.cbxtakeatour_popover_' . $id . ' .btn-secondary:hover {
  color: ' . $button_hover_text_color . ';
  background-color: ' . $button_hover_color . ';
  border-color: ' . $button_hover_border_color . ';
}
.cbxtakeatour_popover_' . $id . ' .btn-secondary.disabled:hover {
	border: 1px solid ' . $button_disabled_hover_border_color . ';
}
.cbxtakeatour_popover_' . $id . ' .btn-secondary.disabled, .cbxtakeatour_popover_' . $id . ' .btn-secondary:disabled {
	
  background-color: ' . $button_color . ';
  border-color: ' . $button_border_color . ';
}
.cbxtakeatour_popover_' . $id . ' .btn-secondary:active, .cbxtakeatour_popover_' . $id . ' .btn-secondary.active,
.cbxtakeatour_popover_' . $id . ' .show > .btn-secondary.dropdown-toggle {
  background-color: ' . $button_active_color . ';
  border-color: ' . $button_active_border_color . ';
}

.cbxtakeatour_popover_' . $id . ' .btn-secondary:focus, .cbxtakeatour_popover_' . $id . ' .btn-secondary.focus {
	box-shadow: 0 0 0 3px ' . $button_focus_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-top .cbxtatarrow::before, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=top] .cbxtatarrow::before {
  border-top-color: ' . $arrow_border_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-top .cbxtatarrow::after, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=top] .cbxtatarrow::after {
  border-top-color: ' . $arrow_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-right .cbxtatarrow::before, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=right] .cbxtatarrow::before {
  border-right-color: ' . $arrow_border_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-right .cbxtatarrow::after, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=right] .cbxtatarrow::after {
  border-right-color: ' . $arrow_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-bottom .cbxtatarrow::before, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=bottom] .cbxtatarrow::before {
  border-bottom-color: ' . $arrow_border_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-bottom .cbxtatarrow::after, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=bottom] .cbxtatarrow::after {
  border-bottom-color: ' . $arrow_color . ';
}



.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-left .cbxtatarrow::before, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=left] .cbxtatarrow::before {
  border-left-color: ' . $arrow_border_color . ';
}

.cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-left .cbxtatarrow::after, .cbxtakeatour_popover_' . $id . '.bs-cbxtakeatour_popover-auto[x-placement^=left] .cbxtatarrow::after {
  border-left-color: ' . $arrow_color . ';
}';

		//pophover end

		return $custom_css;
	}//end add_custom_css

	/**
	 * Char Length check  thinking utf8 in mind
	 *
	 * @param $text
	 *
	 * @return int
	 */
	public static function utf8_compatible_length_check( $text ) {
		if ( seems_utf8( $text ) ) {
			$length = mb_strlen( $text );
		} else {
			$length = strlen( $text );
		}

		return $length;
	}

	/**
	 * Setup a post object and store the original loop item so we can reset it later
	 *
	 * @param obj $post_to_setup The post that we want to use from our custom loop
	 */
	public static function setup_admin_postdata( $post_to_setup ) {
		//only on the admin side
		if ( is_admin() ) {

			//get the post for both setup_postdata() and to be cached
			global $post;

			//only cache $post the first time through the loop
			if ( ! isset( $GLOBALS['post_cache'] ) ) {
				$GLOBALS['post_cache'] = $post;
			}

			//setup the post data as usual
			$post = $post_to_setup;
			setup_postdata( $post );
		} else {
			setup_postdata( $post_to_setup );
		}
	}//end setup_admin_postdata


	/**
	 * Reset $post back to the original item
	 */
	public static function wp_reset_admin_postdata() {
		//only on the admin and if post_cache is set
		if ( is_admin() && ! empty( $GLOBALS['post_cache'] ) ) {

			//globalize post as usual
			global $post;

			//set $post back to the cached version and set it up
			$post = $GLOBALS['post_cache'];
			setup_postdata( $post );

			//cleanup
			unset( $GLOBALS['post_cache'] );
		}

		if ( ! is_admin() ) {
			wp_reset_postdata();
		}
	}//end wp_reset_admin_postdata

	/**
	 * Add utm params to any url
	 *
	 * @param string $url
	 *
	 * @return string
	 */
	public static function url_utmy( $url = '' ) {
		if ( $url == '' ) {
			return $url;
		}

		$url = add_query_arg( array(
			'utm_source'   => 'plgsidebarinfo',
			'utm_medium'   => 'plgsidebar',
			'utm_campaign' => 'wpfreemium',
		), $url );

		return $url;
	}//end url_utmy

	/**
	 * Get all  core tables list
	 */
	public static function getAllDBTablesList() {
		global $wpdb;

		$table_names                  = array();



		return apply_filters( 'cbxtakeatour_table_list', $table_names );
	}//end method getAllDBTablesList

	/**
	 * List all global option name with prefix cbxtakeatour_
	 */
	public static function getAllOptionNames() {
		global $wpdb;

		$prefix       = 'cbxtakeatour_';
		$option_names = $wpdb->get_results( "SELECT * FROM {$wpdb->options} WHERE option_name LIKE '{$prefix}%'", ARRAY_A );

		return apply_filters( 'cbxtakeatour_option_names', $option_names );
	}//end method getAllOptionNames

}//end class CBXTakeaTourHelper