<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXTakeaTour
 * @subpackage CBXTakeaTour/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    CBXTakeaTour
 * @subpackage CBXTakeaTour/public
 * @author     Codeboxr Team <sabuj@codeboxr.com>
 */
class CBXTakeaTour_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name      = $plugin_name;
		$this->version          = $version;
		$this->settings_api     = new CBXTakeaTour_Settings( );
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_register_style( 'bootstrap-tour-standalone',
			plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone.css',
			array(),
			$this->version,
			'all' );
		/*wp_register_style( 'bootstrap-tour-standalone-red',
			plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone_red.css',
			array(),
			$this->version,
			'all' );*/

		wp_register_style( 'cbxtakeatour-public',
			plugin_dir_url( __FILE__ ) . '../assets/css/cbxtakeatour-public.css',
			array( 'bootstrap-tour-standalone' ),
			$this->version,
			'all' );

		wp_enqueue_style( 'bootstrap-tour-standalone' );
		wp_enqueue_style( 'cbxtakeatour-public' );
	}//end enqueue_styles

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_register_script( 'bootstrap-tour-standalone',
			plugin_dir_url( __FILE__ ) . '../assets/js/bootstrap-tour-standalone.js',
			array( 'jquery' ),
			$this->version,
			true );


		wp_register_script( 'cbxtakeatour-events', plugin_dir_url( __FILE__ ) . '../assets/js/cbxtakeatour-events.js', array(), $this->version, true );
		wp_register_script( "cbxtakeatour-public",
			plugin_dir_url( __FILE__ ) . '../assets/js/cbxtakeatour-public.js',
			array( 'cbxtakeatour-events', 'jquery', 'bootstrap-tour-standalone' ),
			$this->version,
			true );


		$cbxtakeatour_js_var = array(
			'steps'             => array(),
			'tour_label_pause'  => esc_html__( 'Pause', 'cbxtakeatour' ),
			'tour_label_resume' => esc_html__( 'Resume', 'cbxtakeatour' )
		);

		wp_localize_script( 'cbxtakeatour-public', 'cbxtakeatour', apply_filters('cbxtakeatour_js_var', $cbxtakeatour_js_var) );

		wp_enqueue_script( 'bootstrap-tour-standalone' );

		wp_enqueue_script( 'cbxtakeatour-events' );

		do_action( 'cbxtakeatour_js_before_public' );

		wp_enqueue_script( 'cbxtakeatour-public' );
	}//end enqueue_scripts

	/**
	 * This method add shortcode and decides the name od shortcode
	 */
	public function init_shortode() {
		add_shortcode( 'cbxtakeatour', array( $this, 'cbxtakeatour_shortcode' ) );
	} // end of init_shortode

	/**
	 * This method holds the all data like how shortcode would be
	 *
	 * @param $atts
	 *
	 * @return string
	 */
	public function cbxtakeatour_shortcode( $atts ) {
		$atts = shortcode_atts( array(
			'id'          => 0,
			'button_text' => '',
			'align'       => '', //'left', 'center', 'right', 'none', put empty to ignore shortcode param
			'block'       => '', //1 = block/full width ,  0 = not applicable , put empty to ignore shortcode param
			'display'     => '', //1 = display , 0 = hide , put empty to ignore shortcode param
			'auto_start'  => ''// 1  = auto start, 0 = starts on click, put empty to ignore shortcode param
		),
			$atts,
			'cbxtakeatour' );



		$id = isset( $atts['id'] ) ? intval( $atts['id'] ) : 0;
		if ( $id == 0 ) {
			return '';
		}

		$id = intval( $id );

		$tour_data = get_post_meta( $id, '_cbxtourmeta', true );


		if ( ! empty( $atts['button_text'] ) ) {
			$tour_data['tour_button_text'] = esc_attr( wp_unslash( $atts['button_text'] ) );
		}

		if ( ! empty( $atts['align'] ) ) {
			$tour_data['tour_button_align'] = esc_attr( wp_unslash( $atts['align'] ) );
		}

		if ( $atts['block'] != '' ) {
			$tour_data['tour_button_block'] = intval( $atts['block'] );
		}

		if ( $atts['display'] != '' ) {
			$tour_data['display'] = intval( $atts['display'] );
		}

		if ( $atts['auto_start'] != '' ) {
			$tour_data['auto_start'] = intval( $atts['auto_start'] );
		}

		$tour_data['id'] = intval( $id );

		return CBXTakeaTourHelper::display_tour( apply_filters('cbxtakeatour_tour_data',  $tour_data, $atts, $id));
	}// end of cbxtakeatour_shortcode

	/**
	 * Classic widget
	 */
	public function register_widget() {
		if ( ! class_exists( 'CBXTakeaTourHelper' ) ) {
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbxtakeatour-helper.php';
		}

		if ( ! class_exists( 'CBXTakeaTour_Widget' ) ) {
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'widgets/classic-widget/class-cbxtakeatour-widget.php';
		}

		register_widget( "CBXTakeaTour_Widget" );
	}//end register_widget

	/**
	 * Init elementor widget
	 *
	 * @throws Exception
	 */
	public function init_elementor_widgets() {
		//include the file
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'widgets/elementor-widget/class-cbxtakeatour-elemwidget.php';

		//register the widget
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new CBXTakeaTourElemWidget\Widgets\CBXTakeaTour_ElemWidget() );
	}//end widgets_registered

	/**
	 * Add new category to elementor
	 *
	 * @param $elements_manager
	 */
	public function add_elementor_widget_categories( $elements_manager ) {
		$elements_manager->add_category(
			'codeboxr',
			array(
				'title' => esc_html__( 'Codeboxr Widgets', 'cbxtakeatour' ),
				'icon'  => 'fa fa-plug',
			)
		);
	}//end add_elementor_widget_categories

	/**
	 * Load Elementor Custom Icon
	 */
	public function elementor_icon_loader() {
		wp_register_style( 'cbxtakeatour_elementor_icon',
			CBXTAKEATOUR_ROOT_URL . 'widgets/elementor-widget/elementor-icon/icon.css', false, $this->version );
		wp_enqueue_style( 'cbxtakeatour_elementor_icon' );

		/*wp_register_style( 'bootstrap-tour-standalone',
			plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone.css',
			array(),
			$this->version,
			'all' );
		wp_register_style( 'bootstrap-tour-standalone-red',
			plugin_dir_url( __FILE__ ) . '../assets/css/bootstrap-tour-standalone_red.css',
			array(),
			$this->version,
			'all' );

		wp_register_style( 'cbxtakeatour-public',
			plugin_dir_url( __FILE__ ) . '../assets/css/cbxtakeatour-public.css',
			array( 'bootstrap-tour-standalone' ),
			$this->version,
			'all' );

		wp_enqueue_style( 'bootstrap-tour-standalone' );
		wp_enqueue_style( 'cbxtakeatour-public' );*/

	}//end elementor_icon_loader

	/**
	 * Before VC Init
	 */
	public function vc_before_init_actions() {
		if ( ! class_exists( 'CBXTakeaTour_WPBWidget' ) ) {
			require_once CBXTAKEATOUR_ROOT_PATH . 'widgets/vc-element/class-cbxtakeatour-wpbwidget.php';
		}

		new CBXTakeaTour_WPBWidget();
	}// end method vc_before_init_actions
}//end class CBXTakeaTour_Public
